@@ -77,7 +77,8 @@ class ApoJWT():
                     raise TypeError("ApoJWT requires the token_finder attribute to be defined for validating JWTs")
                 token = self.__token_f(*args, **kwargs)
                 token_dict = self.__decode(token, *args, **kwargs)
-                self.__form_args(fn, token_dict["data"], token_dict["sub"], kwargs)
+                kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                self.__form_kwargs(fn, kwarg_dict, kwargs)
                 return await fn(*args, **kwargs)
             return wrapper
         else:
@@ -87,7 +88,8 @@ class ApoJWT():
                     raise TypeError("ApoJWT requires the token_finder attribute to be defined for validating JWTs")
                 token = self.__token_f(*args, **kwargs)
                 token_dict = self.__decode(token, *args, **kwargs)
-                self.__form_args(fn, token_dict["data"], token_dict["sub"], kwargs)
+                kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                self.__form_kwargs(fn, kwarg_dict, kwargs)
                 return fn(*args, **kwargs)
             return wrapper
         
@@ -113,7 +115,8 @@ class ApoJWT():
                     token_dict = self.__decode(token, *args, **kwargs)
                     decoded_permissions = token_dict["perms"]
                     if self.__admin_perm is not None and self.__admin_perm in decoded_permissions:
-                        self.__form_args(fn, token_dict["data"], token_dict["sub"], kwargs)
+                        kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                        self.__form_kwargs(fn, kwarg_dict, kwargs)
                         return await fn(*args, **kwargs)
 
                     formatted_perm = permission if self.__permission_formatter is None else self.__permission_formatter(permission=permission, *args, **kwargs)                        
@@ -121,9 +124,12 @@ class ApoJWT():
                         if self.__exception_handler is None:
                             raise jwt.exceptions.InvalidTokenError(f"403: JWT is not authorized for this action")
                         else:
-                            self.__exception_handler(code=403, exception_type="unauthorized", msg="JWT is not authorized for this action", *args, **kwargs)
+                            kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=403, exception_type="unauthorized", msg="JWT is not authorized for this action")
+                            self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
+                            self.__exception_handler(*args, **kwargs)
                             raise NotImplementedError("ApoJWT Exception Handler must not return control")
-                    self.__form_args(fn, token_dict["data"], token_dict["sub"], kwargs)
+                    kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                    self.__form_kwargs(fn, kwarg_dict, kwargs)
                     return await fn(*args, **kwargs)
                 return wrapper
             else: 
@@ -135,7 +141,8 @@ class ApoJWT():
                     token_dict = self.__decode(token, *args, **kwargs)
                     decoded_permissions = token_dict["perms"]
                     if self.__admin_perm is not None and self.__admin_perm in decoded_permissions:
-                        self.__form_args(fn, token_dict["data"], token_dict["sub"], kwargs)
+                        kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                        self.__form_kwargs(fn, kwarg_dict, kwargs)
                         return fn(*args, **kwargs)
 
                     formatted_perm = permission if self.__permission_formatter is None else self.__permission_formatter(permission, *args, **kwargs)
@@ -145,7 +152,8 @@ class ApoJWT():
                         else:
                             self.__exception_handler(code=403, exception_type="unauthorized", msg="JWT is not authorized for this action", *args, **kwargs)
                             raise NotImplementedError("ApoJWT Exception Handler must not return control")
-                    self.__form_args(fn, token_dict["data"], token_dict["sub"], kwargs)
+                    kwarg_dict = dict(token_data=token_dict["data"], token_subject=token_dict["sub"])
+                    self.__form_kwargs(fn, kwarg_dict, kwargs)
                     return fn(*args, **kwargs)
                 return wrapper
         return permission_decorated
@@ -160,13 +168,19 @@ class ApoJWT():
                     raise NotImplementedError("ApoJWT requires the token_finder attribute to be defined for refreshing JWTs")
                 if self.__refresh_f is None:
                     raise NotImplementedError("ApoJWT requires the refresh_finder attribute to be defined for refreshing JWTs")
+                if "token_subject" in kwargs.keys():
+                    kwargs.pop("token_subject")
+                if "token_data" in kwargs.keys():
+                    kwargs.pop("token_data")
                 ref_token = self.__refresh_f(*args, **kwargs)
-                jwt.decode(ref_token, self.__refresh_secret, issuer=self.iss, audience="default", algorithms=[self.algorithm])
+                self.__decode(ref_token, secret=self.__refresh_secret, *args, **kwargs)
                 acc_token = self.__token_f(*args, **kwargs)
                 token_payload = self.__decode(acc_token, verify_exp=False, *args, **kwargs)
                 token_payload["exp"] = int(time()) + self.exp_period
                 new_token = jwt.encode(token_payload, self.__secret, algorithm=self.algorithm)
-                return await fn(access_token=new_token, *args, **kwargs)
+                kwarg_dict = dict(token_data=token_payload["data"], token_subject=token_payload["sub"], access_token=new_token)
+                self.__form_kwargs(fn, kwarg_dict, kwargs)
+                return await fn(*args, **kwargs)
             return wrapper
         else:
             @wraps(fn)
@@ -175,13 +189,19 @@ class ApoJWT():
                     raise NotImplementedError("ApoJWT requires the token_finder attribute to be defined for refreshing JWTs")
                 if self.__refresh_f is None:
                     raise NotImplementedError("ApoJWT requires the refresh_finder attribute to be defined for refreshing JWTs")
+                if "token_subject" in kwargs.keys():
+                    kwargs.pop("token_subject")
+                if "token_data" in kwargs.keys():
+                    kwargs.pop("token_data")
                 ref_token = self.__refresh_f(*args, **kwargs)
                 self.__decode(ref_token, secret=self.__refresh_secret, *args, **kwargs)
                 acc_token = self.__token_f(*args, **kwargs)
                 token_payload = self.__decode(acc_token, verify_exp=False, *args, **kwargs)
                 token_payload["exp"] = int(time()) + self.exp_period
                 new_token = jwt.encode(token_payload, self.__secret, algorithm=self.algorithm)
-                return fn(access_token=new_token, *args, **kwargs)
+                kwarg_dict = dict(token_data=token_payload["data"], token_subject=token_payload["sub"], access_token=new_token)
+                self.__form_kwargs(fn, kwarg_dict, kwargs)
+                return fn(*args, **kwargs)
             return wrapper
         
 
@@ -231,9 +251,6 @@ class ApoJWT():
         return access_token
 
 
-
-
-
     def __decode(self, token, secret=None, verify_exp=True, *args, **kwargs):
         """Decodes the active jwt and returns the result"""
         not_implemented = NotImplementedError("ApoJWT Exception Handler must not return control")
@@ -246,43 +263,47 @@ class ApoJWT():
             if self.__exception_handler is None:
                 raise jwt.exceptions.InvalidSignatureError(f"401: JWT signature is invalid")
             else:
-                self.__exception_handler(code=401, exception_type="signature", msg="JWT signature is invalid", *args, **kwargs)
+                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, exception_type="signature", msg="JWT signature is invalid")
+                self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
+                self.__exception_handler(*args, **kwargs)
                 raise not_implemented
         except jwt.exceptions.ExpiredSignatureError:
             if self.__exception_handler is None:
                 raise jwt.exceptions.ExpiredSignatureError(f"403: JWT signature has expired")
             else:
-                self.__exception_handler(code=403, exception_type="expired", msg="JWT signature has expired", *args, **kwargs)
+                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=403, exception_type="expired", msg="JWT signature has expired")
+                self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
+                self.__exception_handler(*args, **kwargs)
                 raise not_implemented
         except jwt.exceptions.InvalidIssuerError:
             if self.__exception_handler is None:
                 raise jwt.exceptions.InvalidIssuerError(f"401: JWT issuer is invalid")
             else:
-                self.__exception_handler(code=401, exception_type="issuer", msg="JWT issuer is invalid", *args, **kwargs)
+                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, exception_type="issuer", msg="JWT issuer is invalid")
+                self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
+                self.__exception_handler(*args, **kwargs)
                 raise not_implemented
         except jwt.exceptions.InvalidAudienceError:
             if self.__exception_handler is None:
                 raise jwt.exceptions.InvalidAudienceError(f"401: JWT audience is invalid")
             else:
-                self.__exception_handler(code=401, exception_type="audience", msg="JWT audience is invalid", *args, **kwargs)
+                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, exception_type="audience", msg="JWT audience is invalid")
+                self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
+                self.__exception_handler(*args, **kwargs)
                 raise not_implemented
         except jwt.exceptions.InvalidTokenError:
             if self.__exception_handler is None:
                 raise jwt.exceptions.InvalidTokenError(f"401: JWT is invalid")
             else:
-                self.__exception_handler(code=401, exception_type="invalid", msg="JWT is invalid", *args, **kwargs)
+                kwarg_dict = dict(token_data=dict(), token_subject="", access_token="", code=401, exception_type="invalid", msg="JWT is invalid")
+                self.__form_kwargs(self.__exception_handler, kwarg_dict, kwargs)
+                self.__exception_handler(*args, **kwargs)
                 raise not_implemented
 
 
-    def __form_args(self, fn, token_data, token_subject, kwargs):
-        """Inspects the function 'fn' and decides if token_data and/or token_subject should be sent as keyword arguments"""
+    def __form_kwargs(self, fn, new_kwargs: dict, kwargs):
+        """Inspects the function 'fn' and decides which new_kwargs should be sent as keyword arguments"""
         fn_args = list(inspect.signature(fn).parameters.keys())
-        td = "token_data" in fn_args
-        ts = "token_subject" in fn_args
-        if td and ts:
-            kwargs["token_data"] = token_data
-            kwargs["token_subject"] = token_subject
-        elif td:
-            kwargs["token_data"] = token_data
-        elif ts:
-            kwargs["token_subject"] = token_subject
\ No newline at end of file
+        for key in new_kwargs.keys():
+            if key in fn_args:
+                kwargs[key] = new_kwargs[key]
\ No newline at end of file
